package unitTest;

import static org.junit.jupiter.api.Assertions.assertFalse;
import static org.junit.jupiter.api.Assertions.assertThrows;
import static org.junit.jupiter.api.Assertions.assertTrue;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.Collection;
import java.util.Iterator;
import java.util.Random;
import java.util.function.Function;
import java.util.stream.Stream;

import org.junit.jupiter.api.BeforeEach;
import org.junit.jupiter.api.DynamicTest;
import org.junit.jupiter.api.Test;
import org.junit.jupiter.api.TestFactory;
import org.junit.jupiter.api.function.Executable;
import org.junit.jupiter.api.function.ThrowingConsumer;
import org.junit.jupiter.params.ParameterizedTest;
import org.junit.jupiter.params.provider.ValueSource;

import exceptions.CategoryInvalidException;
import exceptions.DescriptionInvalidException;
import exceptions.ImageInvalidException;
import exceptions.TitleInvalidException;
import project.Auction;
import project.Category;
import project.Item;
import project.ItemPicture;
import project.User;

public class ItemUnitTest {

	@BeforeEach
	public void reset() {

		Item.items = new ArrayList<Item>();
		User.users = new ArrayList<User>();
		Auction.auctions = new ArrayList<Auction>();
		Category.categories = new ArrayList<Category>();

	}

	@TestFactory
	public Stream<DynamicTest> deleteTestTrue() {

		Iterator<Integer> inputGenerator = new Iterator<Integer>() {

			int itemId;
			Random random = new Random();

			@Override
			public boolean hasNext() {

				return random.nextInt(100) % 10 != 0;

			}

			@Override
			public Integer next() {

				itemId = (int) (random.nextDouble() * 10);
				Item i = new Item();
				i.setId(itemId);
				return itemId;
			}

		};

		Function<Integer, String> displayNameGenerator = (input) -> "item ID: " + input;

		ThrowingConsumer<Integer> testExecutor = (input) -> assertTrue(Item.delete(input));

		return DynamicTest.stream(inputGenerator, displayNameGenerator, testExecutor);

	}

	@TestFactory
	public Stream<DynamicTest> deleteTestFalse() {

		Iterator<Integer> inputGenerator = new Iterator<Integer>() {

			int itemId;
			Random random = new Random();

			@Override
			public boolean hasNext() {

				return random.nextInt(100) % 10 != 0;

			}

			@Override
			public Integer next() {

				itemId = (int) (random.nextDouble() * 10);
				return itemId;
			}

		};

		Function<Integer, String> displayNameGenerator = (input) -> "item ID: " + input;

		ThrowingConsumer<Integer> testExecutor = (input) -> assertFalse(Item.delete(input));

		return DynamicTest.stream(inputGenerator, displayNameGenerator, testExecutor);

	}

	@TestFactory
	public Collection<DynamicTest> modifyAcceptanceTest() {

		Collection<DynamicTest> dynamicTests = new ArrayList<DynamicTest>();

		Item item = Item.getInstance();
		item.setTitle("title");
		item.setDescription("description");
		item.setCategory(new Category("camera"));

		ArrayList<String> titles = new ArrayList<String>(Arrays.asList("", "9999", "t-#\\", ""));
		ArrayList<String> titleTestNames = new ArrayList<String>(Arrays.asList("title empty", "numerical title",
				"symbols present in the title", "title longer than 30 characters", "cateogry not correct"));
		ArrayList<Class<? extends Exception>> titleExceptions = new ArrayList<Class<? extends Exception>>(
				Arrays.asList(TitleInvalidException.class, TitleInvalidException.class, TitleInvalidException.class,
						TitleInvalidException.class));

		for (int i = 0; i < titles.size(); i++) {
			String prepareTitle = titles.get(i);

			if (i == 3) {

				for (int k = 0; k < 60; k++) {

					prepareTitle += "a";

				}
			}

			final String title = prepareTitle;
			final int j = i;
			Executable exec = () -> assertThrows(titleExceptions.get(j),
					() -> item.modify(title, "description", "camera"));
			DynamicTest dTest = DynamicTest.dynamicTest(titleTestNames.get(i), exec);

			dynamicTests.add(dTest);

		}

		ArrayList<String> descriptions = new ArrayList<String>(Arrays.asList("", "9999", ""));
		ArrayList<String> descriptionTestNames = new ArrayList<String>(
				Arrays.asList("description empty", "numerical description", "title longer than 500 characters"));
		ArrayList<Class<? extends Exception>> descriptionExceptions = new ArrayList<Class<? extends Exception>>(
				Arrays.asList(DescriptionInvalidException.class, DescriptionInvalidException.class,
						DescriptionInvalidException.class));

		for (int i = 0; i < descriptions.size(); i++) {

			String prepareDescription = descriptions.get(i);

			if (i == 2) {

				for (int k = 0; k < 600; k++) {

					prepareDescription += "a";

				}
			}

			final String description = prepareDescription;
			final int j = i;
			Executable exec = () -> assertThrows(descriptionExceptions.get(j),
					() -> item.modify("title", description, "camera"));
			DynamicTest dTest = DynamicTest.dynamicTest(descriptionTestNames.get(i), exec);

			dynamicTests.add(dTest);

		}

		ArrayList<String> category = new ArrayList<String>(Arrays.asList("bike"));
		ArrayList<String> categoryTestNames = new ArrayList<String>(Arrays.asList("cateogry not correct"));
		ArrayList<Class<? extends Exception>> categoryExceptions = new ArrayList<Class<? extends Exception>>(
				Arrays.asList(CategoryInvalidException.class));

		for (int i = 0; i < category.size(); i++) {

			final int j = i;

			Executable exec = () -> assertThrows(categoryExceptions.get(j),
					() -> item.modify("title", "description", category.get(j)));
			DynamicTest dTest = DynamicTest.dynamicTest(categoryTestNames.get(i), exec);

			dynamicTests.add(dTest);

		}

		return dynamicTests;

	}

	@Test
	public void modifyTestTrue() throws CategoryInvalidException, DescriptionInvalidException, TitleInvalidException {

		Item item = Item.getInstance();
		new Category("camera");

		assertTrue(item.modify("title", "description", "camera"));

	}

	@ParameterizedTest
	@ValueSource(ints = { -1, 20 })
	public void addPictureOutOfBoundTest(int arrayLength) {

		ArrayList<ItemPicture> list = new ArrayList<ItemPicture>();

		for (int e = 0; e < arrayLength; e++) {
			list.add(new ItemPicture());
		}

		Item i = Item.getInstance();
		assertThrows(ImageInvalidException.class, () -> i.addPicture(list));

	}

	@Test
	public void addPictureWorkingTest() throws ImageInvalidException {

		ArrayList<ItemPicture> list = new ArrayList<ItemPicture>();
		int value = 5;

		for (int e = 0; e < value; e++) {

			list.add(new ItemPicture());

		}

		Item i = Item.getInstance();
		assertTrue(i.addPicture(list));

	}

}
