package unitTest;

import static org.junit.jupiter.api.Assertions.assertEquals;
import static org.junit.jupiter.api.Assertions.assertFalse;
import static org.junit.jupiter.api.Assertions.assertNull;
import static org.junit.jupiter.api.Assertions.assertThrows;
import static org.junit.jupiter.api.Assertions.assertTrue;

import java.text.ParseException;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.Collection;
import java.util.Date;

import org.junit.jupiter.api.BeforeEach;
import org.junit.jupiter.api.DynamicTest;
import org.junit.jupiter.api.Test;
import org.junit.jupiter.api.TestFactory;
import org.junit.jupiter.api.function.Executable;
import org.junit.jupiter.params.ParameterizedTest;
import org.junit.jupiter.params.provider.ValueSource;

import exceptions.*;
import project.Auction;
import project.AuctionManager;
import project.Category;
import project.Item;
import project.User;

public class UserUnitTest {

	@BeforeEach
	public void reset() {

		Item.items = new ArrayList<Item>();
		User.users = new ArrayList<User>();
		Auction.auctions = new ArrayList<Auction>();
		Category.categories = new ArrayList<Category>();

	}

	@Test
	public void loginTest()
			throws UsernameEmailEmptyException, PasswordInvalidException, MailInvalidException, LoginInvalidException {

		String username = "mario", password = "password", email = "test@gmail.com";
		User u = new User();
		u.setUsername(username);
		u.setEmail(email);
		u.setPassword(password);

		assertTrue(User.login(username, email, password));
		assertTrue(User.login("", email, password));
		assertThrows(LoginInvalidException.class, () -> User.login(username, email, "wrong password"));
		assertThrows(LoginInvalidException.class, () -> User.login("wrongUsername", email, "wrong password"));
		assertThrows(LoginInvalidException.class, () -> User.login("", "wrongemail@gmail.com", "wrong password"));
		assertThrows(LoginInvalidException.class, () -> User.login("", email, "wrong password"));

	}

	@Test
	public void loginAcceptanceTest() {

		assertThrows(UsernameEmailEmptyException.class, () -> User.login("", "", "test"));
		assertThrows(MailInvalidException.class, () -> User.login("", "an.gmail.it", "test"));
		assertThrows(PasswordInvalidException.class, () -> User.login("", "an.gmail.it", ""));

	}

	@Test
	public void searchItemTest() throws CategoryInvalidException, TitleInvalidException {

		String title = "Red bike", category = "Sport and Outdoor", category2 = "Cars and Motorbikes";
		User u = User.getInstance();
		new Category(category);
		new Category(category2);
		ArrayList<Item> itemResult = new ArrayList<Item>();
		ArrayList<Item> emptyArray = new ArrayList<Item>();

		Item i = new Item();
		i.setTitle(title);
		i.setCategory(Category.getCategory(category));

		itemResult.add(i);

		assertEquals(u.searchItem(title, category), itemResult);
		assertEquals(u.searchItem("ferrari", category), emptyArray);
		assertEquals(u.searchItem(title, category2), emptyArray);
		assertEquals(u.searchItem("ferrari", category2), emptyArray);

	}

	@TestFactory
	public Collection<DynamicTest> searchAuctionAcceptanceTest() {

		Collection<DynamicTest> dynamicTests = new ArrayList<DynamicTest>();
		User u = User.getInstance();
		new Category("camera");

		ArrayList<String> titles = new ArrayList<String>(Arrays.asList("", "9999", "t-#\\", "too long"));
		ArrayList<String> testTitleNames = new ArrayList<String>(
				Arrays.asList("Title empty", "Numeric title", "Symbols in the title", "Title too long"));

		for (int i = 0; i < titles.size(); i++) {

			String prepareTitle = titles.get(i);

			if (i == 3) {

				for (int k = 0; k < 60; k++) {

					prepareTitle += "a";

				}
			}

			final String title = prepareTitle;
			final int j = i;

			Executable exec = () -> assertThrows(TitleInvalidException.class,
					() -> u.searchAuction(5, title, "camera"));
			DynamicTest dTest = DynamicTest.dynamicTest(testTitleNames.get(j), exec);

			dynamicTests.add(dTest);

		}

		ArrayList<String> categories = new ArrayList<String>(Arrays.asList(""));

		for (int i = 0; i < categories.size(); i++) {

			final int j = i;

			Executable exec = () -> assertThrows(CategoryInvalidException.class,
					() -> u.searchAuction(5, "Red Bike", categories.get(j)));
			DynamicTest dTest = DynamicTest.dynamicTest("Category not existent", exec);

			dynamicTests.add(dTest);

		}

		ArrayList<Integer> prices = new ArrayList<Integer>(Arrays.asList(-1, 0));
		ArrayList<String> testPriceNames = new ArrayList<String>(
				Arrays.asList("Negative price", "Price equal to zero"));

		for (int i = 0; i < prices.size(); i++) {

			final int j = i;

			Executable exec = () -> assertThrows(PriceInvalidException.class,
					() -> u.searchAuction(prices.get(j), "Red Bike", "camera"));
			DynamicTest dTest = DynamicTest.dynamicTest(testPriceNames.get(j), exec);

			dynamicTests.add(dTest);

		}

		return dynamicTests;

	}

	@Test
	public void searchAuctionTest()
			throws CategoryInvalidException, TitleInvalidException, PriceInvalidException, ParseException {

		String title = "Red bike", category = "Sport and Outdoor", category2 = "Cars and Motorbikes";
		Date start = null, end = null;

		start = AuctionManager.parseDate("22/06/2020 11:20:12");
		end = AuctionManager.parseDate("22/07/2020 11:20:12");

		double price = 10;
		int bid = 5;
		User u = User.getInstance();
		new Category(category);
		new Category(category2);
		ArrayList<Auction> auctionResult = new ArrayList<Auction>();
		ArrayList<Auction> emptyArray = new ArrayList<Auction>();

		Item i = new Item();
		i.setTitle(title);
		i.setCategory(Category.getCategory(category));

		Auction a = new Auction(i, start, end, price, bid);

		auctionResult.add(a);

		assertEquals(u.searchAuction(price, title, category), auctionResult);
		assertEquals(u.searchAuction(price + 1, title, category), auctionResult);
		assertEquals(u.searchAuction(price, "ferrari", category), emptyArray);
		assertEquals(u.searchAuction(price, title, category2), emptyArray);
		assertEquals(u.searchAuction(price - 1, "ferrari", category2), emptyArray);

	}

	@TestFactory
	public Collection<DynamicTest> searchItemAcceptanceTest() {

		Collection<DynamicTest> dynamicTests = new ArrayList<DynamicTest>();

		new Category("camera");
		User u = User.getInstance();

		ArrayList<String> titles = new ArrayList<String>(Arrays.asList("", "9999", "t-#\\", "too long"));
		ArrayList<String> testTitleNames = new ArrayList<String>(
				Arrays.asList("Title empty", "Numeric title", "Symbols in the title", "Title too long"));

		for (int i = 0; i < titles.size(); i++) {

			String prepareTitle = titles.get(i);

			if (i == 3) {

				for (int k = 0; k < 60; k++) {

					prepareTitle += "a";

				}
			}

			final String title = prepareTitle;
			final int j = i;

			Executable exec = () -> assertThrows(TitleInvalidException.class, () -> u.searchItem(title, "camera"));
			DynamicTest dTest = DynamicTest.dynamicTest(testTitleNames.get(j), exec);

			dynamicTests.add(dTest);

		}

		ArrayList<String> categories = new ArrayList<String>(Arrays.asList("", "bike"));

		for (int i = 0; i < categories.size(); i++) {

			final int j = i;

			Executable exec = () -> assertThrows(CategoryInvalidException.class,
					() -> u.searchAuction(5, "Red Bike", categories.get(j)));
			DynamicTest dTest = DynamicTest.dynamicTest("Category not existent", exec);

			dynamicTests.add(dTest);

		}

		return dynamicTests;

	}

	@ParameterizedTest
	@ValueSource(ints = { 0, -1, -20 })
	public void searchItemError(int userId) {

		String title = "Red Bike";
		String category = "Sport and Outdoor";

		new Category(category);

		Item i = new Item();
		i.setTitle(title);
		i.setCategory(Category.getCategory(category));

		User u = User.getInstance();
		assertThrows(PriceInvalidException.class, () -> u.searchAuction(userId, title, category));

	}

	@Test
	public void addItemTest() {

		int id1 = 1;

		User u = User.getInstance();
		Item i = Item.getInstance();
		i.setId(id1);

		assertTrue(u.addItemToList(id1));
		assertFalse(u.addItemToList(id1));

	}

	@Test
	public void getUserByIdTest() {

		int id1 = 1;

		User u = User.getInstance();
		u.setId(id1);

		assertEquals(u, User.getUserById(id1));
		assertNull(User.getUserById(2));

	}

}
